*parse gazetteer files
*downloaded from : http://www.census.gov/geo/maps-data/data/gazetteer.html
*1) standardize zip code and state abbreviation: zipcode, st_abbrev
*2) standardize latitude and longitude: zip_lat, zip_long

use input/state_fips.dta, clear
rename state_pc st_abbrev
sort st_abbrev
save input/state_fips.dta, replace

insheet using "input/gaz_zips_1990.txt", double clear //n=29470, 6 duplicates zipcodes (3 surplus)
*Zipcodes 32350, 63673, and 42223 are each in two states and have two observations; no other duplicate zipcodes
rename v1 state_fips
label variable state_fips "state FIPS code"
rename v2 zipNum
gen zipcode=string(zipNum,"%05.0f") //give leading zeroes for zip codes with only four digits, use string for zips like "351HH"
rename v3 st_abbrev
rename v4 zip_name
label variable zip_name "Gazetteer name for zip code"
rename v5 zip_long
label variable zip_long "Longitude in Decimal Degrees (West is assumed, no minus sign)"
rename v6 zip_lat
label variable zip_lat "Latitude in Decimal Degrees (North is assumed, no plus sign)"
rename v7 zip_pop
rename v8 zip_alloc
label variable zip_alloc "decimal portion of state within zipcode"
gen year=1990
replace zip_long=-1*zip_long //Western hemisphere longitude should be negative, required for GIS work
sort zipcode state_fips
save "temp\zip_gaz_1990.dta", replace
drop if st_abbrev=="AK"|st_abbrev=="HI" //only continental US
outsheet using "output\zip_gaz_1990.csv", comma replace

*Note: the DCT file looks for gaz_zips_2000.txt file in "input" directory--the data file cannot be moved or code won't work
infix using code\gaz_zip_2000.dct, clear //n=33,233, 108 duplicates due to some ZCTAs being in multiple states
*Next step is to parse the "zip_name" field to separate out zip codes and ZCTA "parts"
gen zipcode=substr(zip_name,1,5) //I leave this as string to accomdate zip codes like "351HH"
rename state st_abbrev
merge m:1 st_abbrev using "input/state_fips.dta", keepusing(state_fips)
drop if _merge==2 //drops 4 obs
drop _merge
sort zipcode
save "temp\zip_gaz_2000.dta", replace
drop if st_abbrev=="AK"|st_abbrev=="HI" //only continental US
sort zipcode state_fips
outsheet using "output\zip_gaz_2000.csv", comma replace

*Format 2010
insheet using "input/gaz_zips_2010.txt", double clear //n=33120, no duplicate zipcodes
gen zipcode=string(geoid,"%05.0f") //give leading zeroes for zip codes with only four digits, use string for zips like "351HH"
rename intptlat zip_lat
rename intptlong zip_long
*No state abbreviation data
merge 1:m zipcode using "temp\zip_gaz_2000.dta", keepusing(st_abbrev zip_pop) //there are duplicates because Gazetteer file splits some zipcodes
drop if _merge==2
duplicates tag zipcode, gen(dupTag)
bysort zipcode: egen maxPop=max(zip_pop) //for duplicates zipcodes in multiple states I keep only zip code with largest 2000 pop
drop if dupTag>0&zip_pop!=maxPop //drops n=43 observations deleted, no more duplicates
drop dupTag _merge maxPop
*Now add in state_fips
merge m:1 st_abbrev using input/state_fips.dta, keepusing(state_fips)
drop if _merge==2 //drops 4 obs
drop _merge
sort zipcode state_fips
save "temp\zip_gaz_2010.dta", replace
drop if st_abbrev=="AK"|st_abbrev=="HI" //only continental US, n=319 dropped
outsheet using "output\zip_gaz_2010.csv", comma replace

*Format 2012
insheet using "input/gaz_zips_2012.txt", double clear //n=33144, no duplicate zipcodes
gen zipcode=string(geoid,"%05.0f") //give leading zeroes for zip codes with only four digits, use string for zips like "351HH"
rename intptlat zip_lat
rename intptlong zip_long
*No state abbreviation data
merge 1:m zipcode using "temp\zip_gaz_2000.dta", keepusing(st_abbrev zip_pop)
drop if _merge==2
duplicates tag zipcode, gen(dupTag)
bysort zipcode: egen maxPop=max(zip_pop) //for duplicates zipcodes in multiple states I keep only zip code with largest 2000 pop
drop if dupTag>0&zip_pop!=maxPop //drops n=43 observations deleted, no more duplicates
drop dupTag _merge maxPop
*Now add in state_fips
merge m:1 st_abbrev using "input/state_fips.dta", keepusing(state_fips)
drop if _merge==2
drop _merge
sort zipcode state_fips
save "temp\zip_gaz_2012.dta", replace
drop if st_abbrev=="AK"|st_abbrev=="HI" //only continental US, n=319 dropped
outsheet using "output\zip_gaz_2012.csv", comma replace

*Format 2013: exactly same as 2012
insheet using "input/gaz_zips_2013.txt", double clear //n=33144, no duplicate zipcodes
gen zipcode=string(geoid,"%05.0f") //give leading zeroes for zip codes with only four digits, use string for zips like "351HH"
rename intptlat zip_lat
rename intptlong zip_long
*No state abbreviation data
merge 1:m zipcode using "temp\zip_gaz_2000.dta", keepusing(st_abbrev zip_pop)
drop if _merge==2
duplicates tag zipcode, gen(dupTag)
bysort zipcode: egen maxPop=max(zip_pop) //for duplicates zipcodes in multiple states I keep only zip code with largest 2000 pop
drop if dupTag>0&zip_pop!=maxPop //drops n=43 observations deleted, no more duplicates
drop dupTag _merge maxPop
*Now add in state_fips
merge m:1 st_abbrev using "input/state_fips.dta", keepusing(state_fips)
drop if _merge==2
drop _merge
sort zipcode state_fips
save "temp\zip_gaz_2013.dta", replace
drop if st_abbrev=="AK"|st_abbrev=="HI" //only continental US, n=319 dropped
outsheet using output\zip_gaz_2013.csv, comma replace

*Format 2014
insheet using "input/gaz_zips_2014.txt", double clear //n=33144, no duplicate zipcodes
gen zipcode=string(geoid,"%05.0f") //give leading zeroes for zip codes with only four digits, use string for zips like "351HH"
rename intptlat zip_lat
rename intptlong zip_long
*No state abbreviation data
merge 1:m zipcode using "temp\zip_gaz_2000.dta", keepusing(st_abbrev zip_pop)
drop if _merge==2
duplicates tag zipcode, gen(dupTag)
bysort zipcode: egen maxPop=max(zip_pop) //for duplicates zipcodes in multiple states I keep only zip code with largest 2000 pop
drop if dupTag>0&zip_pop!=maxPop //drops n=43 observations deleted, no more duplicates
drop dupTag _merge maxPop
*Now add in state_fips
merge m:1 st_abbrev using "input/state_fips.dta", keepusing(state_fips)
drop if _merge==2
drop _merge
sort zipcode state_fips
save "temp\zip_gaz_2014.dta", replace
drop if st_abbrev=="AK"|st_abbrev=="HI" //only continental US, n=319 dropped
outsheet using "output\zip_gaz_2014.csv", comma replace
